//
//  MyScene.m
//  SpriteKitGame
//
//  Created by José María Martín Luque on 29/09/13.
//  Copyright (c) 2013 Flip Flop Software. All rights reserved.
//

#import "MyScene.h"
#import "GameOver.h"

@implementation MyScene

-(id)initWithSize:(CGSize)size {    
    if (self = [super initWithSize:size]) {
        /* Setup your scene here */
        
        //Initialises the main variables the game uses
        isDamaged = NO;
        score = 0;
        speed = 10;
        
        //sets the background color
        self.backgroundColor = [SKColor whiteColor];
        
        //creates the game label at the playing screen
        [self addChild:[self createLabelWithTitle:@"The Ball Game"]];
        //creates the "basket"
        [self addChild:[self createBasket]];
        //creates the score label
        [self addChild:[self createScoreLabel:[theScore stringValue]]];
        
        //creates the red balls
        [self performSelector:@selector(createBadBall) withObject:nil afterDelay:2];
        //creates the blue balls
        [self performSelector:@selector(createGoodBall) withObject:nil afterDelay:2];
        //increases the speed gradually
        [self performSelector:@selector(increaseSpeed) withObject:nil afterDelay:1];
        
    }
    return self;
}

-(SKLabelNode*) createLabelWithTitle:(NSString*)theTitle{
    
    //allows the creation ob labels, in this case, the game title label
    
    SKLabelNode *myLabel = [SKLabelNode labelNodeWithFontNamed:@"System Font"];
    
    myLabel.text = theTitle;
    myLabel.fontColor = [SKColor blackColor];
    myLabel.fontSize = 40;
    myLabel.position = CGPointMake(CGRectGetMidX(self.frame), 912);
    
    return myLabel;
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    /* Called when a touch begins */
    
    UITouch *touch = [touches anyObject];
    CGPoint positionInScene = [touch locationInNode:self];
    [self selectNodeForTouch:positionInScene];
}

- (SKSpriteNode*) createBasket {

    //Creates the green basket, the main "character"
    
    CGPoint greenBasketInitPoint = CGPointMake(CGRectGetMidX(self.frame), 200);
    SKSpriteNode *greenBasket = [SKSpriteNode spriteNodeWithImageNamed:@"GreenBasket"];
    
    greenBasket.position = greenBasketInitPoint;
    
    greenBasket.name = @"greenBasket";
    
    return greenBasket;

}

- (void) createBadBall {

    //Creates the red balls that fall from the sky
    
    //defines a random point in which the ball will fall
    CGPoint badBallInitPoint = CGPointMake(arc4random_uniform(768), 900);
    //Creates the sprite itself for the ball and gives it an image
    SKSpriteNode *badBall = [SKSpriteNode spriteNodeWithImageNamed:@"BadBall"];
    
    //sets the node position to the random point we defined earlier
    badBall.position = badBallInitPoint;
    //sets the node name
    badBall.name = @"badBall";
    
    //adds the node
    [self addChild:badBall];
    
    //creates a random appear rate
    float badBallAppearRate = arc4random_uniform(2) + 1;
    //the method calls itself
    [self performSelector:@selector(createBadBall) withObject:nil afterDelay:badBallAppearRate];

}

- (void) createGoodBall {
    
    //Creates the blue balls that fall from the sky
    
    //defines a random point in which the ball will fall
    CGPoint goodBallInitPoint = CGPointMake(arc4random_uniform(768), 900);
    //Creates the sprite itself for the ball and gives it an image
    SKSpriteNode *goodBall = [SKSpriteNode spriteNodeWithImageNamed:@"GoodBall"];
    
    //sets the position I defined earlier as the initial point for the ball
    goodBall.position = goodBallInitPoint;
    //sets the node name for later reference
    goodBall.name = @"goodBall";
    
    //adds the node to the game
    [self addChild:goodBall];
    
    //creates a random number for the appear rate
    float goodBallAppearRate = arc4random_uniform(2) + 1;
    
    //the method calls itself to create balls
    [self performSelector:@selector(createGoodBall) withObject:nil afterDelay:goodBallAppearRate];


}

- (void) selectNodeForTouch:(CGPoint)touchLocation {

    //This is the code that allows the selection of the basket as the node we're going to move
    SKSpriteNode *touchedNode = (SKSpriteNode *)[self nodeAtPoint:touchLocation];
    
        //2
    if (![_selectedNode isEqual:touchedNode]) {
        [_selectedNode removeAllActions];
        [_selectedNode runAction:[SKAction rotateToAngle:0.0f duration:0.1]];
        
        _selectedNode = touchedNode;
        
        }
                                  
    };




- (void)panForTranslation:(CGPoint)translation {
    
    //Allows the basket to be moved
    
    CGPoint position = [_selectedNode position];
    if([[_selectedNode name] isEqualToString:@"greenBasket"]) {
        [_selectedNode setPosition:CGPointMake(position.x + translation.x, position.y)];
    }
    
    }

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    
    //Detects if we moved the finger across the screen
    
	UITouch *touch = [touches anyObject];
	CGPoint positionInScene = [touch locationInNode:self];
	CGPoint previousPosition = [touch previousLocationInNode:self];
    
	CGPoint translation = CGPointMake(positionInScene.x - previousPosition.x, positionInScene.y - previousPosition.y);
    
	[self panForTranslation:translation];
}


-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {

    

}

-(void)update:(CFTimeInterval)currentTime {
    
    //Everything here will execute every frame
    
    //This calls the "basket"
    SKNode *basket = [self childNodeWithName:@"greenBasket"];
    
    //Asigns theScore the value of score for later displaying it
    theScore = [NSNumber numberWithInt: score];
    
    //Calls the method that changes the score label to reflect changes
    [self changeScoreLabel];
    
    
    
    
    [self enumerateChildNodesWithName:@"badBall" usingBlock:^(SKNode *node, BOOL *stop) {
        
        //if there are any badball, this will execute
        
        if (node.position.x < 0 || node.position.y < 0) {
            
            //If the ball has reached the bottom and it doesn't appear on the screen, it will be removed
            
            [node removeFromParent];
            
        } else {
        
            //Else, if there is a ball in the screen, it will be pushed to the bottom with the variable speed, which changes over time
            
            node.position = CGPointMake(node.position.x, node.position.y - speed);
        
        }
        
    
        
        
        //Fuera de ese if
        
        if ([basket intersectsNode:node]&& isDamaged == NO){
        
            //If the basket interscts the node (badBall) it will add one damage point by calling the doDamage method
            
            [self doDamage];
            
            //Prints log to the console, for debugging pourposes
            NSLog(@"Intersection");
        }
        
    }];
    
    [self enumerateChildNodesWithName:@"goodBall" usingBlock:^(SKNode *node, BOOL *stop) {
        
        //if there are any goodBall, this will execute
        
        if (node.position.x < 0 || node.position.y < 0) {
            
            //If the ball has reached the bottom and it doesn't appear on the screen, it will be removed
            
            [node removeFromParent];
            
        } else {
            
            //Else, if there is a ball in the screen, it will be pushed to the bottom with the variable speed, which changes over time
            
            node.position = CGPointMake(node.position.x, node.position.y - speed);
            
        }
        
        if ([basket intersectsNode:node]){
            
            //If the basket intersects the node (goodBall) it will add the score
            
            [self addScore];
            //Prints log to the console, for debugging pourposes
            NSLog(@"Puntuación: %i", score);
        }
        
        }];
}


- (void) doDamage {

    //Sets the isDamaged value to yes and ads a hit point
    
    isDamaged = YES;
    hitCount++;
    
    //calls the damageDone method after a second to avoid being damaged all the frames the badBall touches the basket
    [self performSelector:@selector(damageDone) withObject:nil afterDelay:1];

}


- (void) damageDone {

    //sets the isDamaged value to NO
    isDamaged = NO;
    
    //If the hitCount is = 3 the game will end, calling the gameOver method
    if (hitCount == 3) {
    
        [self gameOver];
    
    }

}

- (void) gameOver {

    //The gameOver method will replace the current scene with the gameOver scene with a push transition
    
    SKScene *gameOverScene = [[GameOver alloc] initWithSize:(self.size)];
    SKTransition *push = [SKTransition pushWithDirection:SKTransitionDirectionUp duration:1];
    [self.view presentScene:gameOverScene transition:push];

}

- (int) increaseSpeed{

    //This is the method used to increase the speed by 1 each 2 seconds
    
    speed = speed + 1;
    [self performSelector:@selector(increaseSpeed) withObject:nil afterDelay:2];
    return speed;
    

}

- (SKLabelNode*) createScoreLabel:(NSString*)scoreTitle{

    //This is the method used to create the score label, similar to the tile one
    
    myScore = [SKLabelNode labelNodeWithFontNamed:@"System Font"];
    
    
    
    myScore.fontColor = [SKColor blackColor];
    myScore.fontSize = 30;
    myScore.position = CGPointMake(CGRectGetMidX(self.frame), 25 );
    myScore.name = @"score";
    
    
    
    return myScore;

}

- (SKLabelNode*) changeScoreLabel {
    
    //Method used to change the scoreLabel to the current score value
    
    myScore.text = [NSString stringWithFormat:@"Score: %@", [theScore stringValue]];
    return myScore;

}

- (int) addScore {
    
    //Method used to increment the score by one (each frame the goodBall touches the basekt for now).

    score++;
    return score;

}






@end
